import 'dart:convert';
import 'dart:io';

import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:checkout/constants/colors.dart';
import 'package:checkout/constants/images.dart';
import 'package:checkout/database/db.dart';
import 'package:checkout/functions/toasts.dart';
import 'package:checkout/functions/url_launcher_function.dart';
import 'package:checkout/models/config_data.dart';
import 'package:checkout/models/self_data.dart';
import 'package:checkout/widgets/button_Global.dart';
import 'package:hive/hive.dart';
import 'package:http/http.dart' as http;
import 'package:loader_overlay/src/overlay_controller_widget_extension.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter_stripe/flutter_stripe.dart';
import 'package:oauth2/oauth2.dart' as oauth2;

class CheckOutScreen extends StatefulWidget {
  const CheckOutScreen({Key? key}) : super(key: key);

  @override
  _CheckOutScreenState createState() => _CheckOutScreenState();
}

class _CheckOutScreenState extends State<CheckOutScreen> {
  Box<configData>? configDataBox;
  configData? config_data;
  Box<SelfData>? SelfDataBox;
  SelfData? self_data;
  bool isloading = true;
  num cart_total = 0;
  num shipping_charges = 0;
  num discount = 0;
  num grand_total = 0;
  String? currency_code;

  String? currency_symbol;
  bool checkout_loading = false;
  bool isAddresSelected = false;
  String? Paymentmethod;

  String? city = "";
  String? state = "";
  String? street = "";
  String? country = "";
  var selected_payment_method;
  var storegence_marketplace;

  void getHiveData(BuildContext context) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var isAddressSelected = await prefs.getBool("isAddressSaved");

    if (isAddressSelected == null || isAddressSelected == false) {
      setState(() {
        isAddresSelected = false;
      });
    } else {
      var street_ = await prefs.getString("street");
      var city_ = await prefs.getString("city");
      var state_ = await prefs.getString("state");
      var country_ = await prefs.getString("country");

      setState(() {
        isAddresSelected = true;
        street = street_;
        city = city_;
        state = state_;
        country = country_;
      });
    }

    var SelectedPaymentMethodString =
        await prefs.getString("selected_payment_method");
    if (SelectedPaymentMethodString != null) {
      var SelectedPaymentMethodJson = jsonDecode(SelectedPaymentMethodString);
      var SelectedPaymentMethod =
          Payment_Methods.fromJson(SelectedPaymentMethodJson);

      setState(() {
        selected_payment_method = SelectedPaymentMethod;
      });
    }

    configDataBox = Hive.box('configDataBox');
    // var box = await Hive.box('configDataBox');
    setState(() {
      config_data = configDataBox?.get('configData');
    });

    SelfDataBox = Hive.box('selfDataBox');

    print("self data" + self_data.toString());

    setState(() {
      self_data = SelfDataBox?.get('selfData');
    });
    // context.loaderOverlay.show();

    await createProfile();
    await getcartTotal();
    await shipping_payment_ids();
    await save_customer_ids();
    // context.loaderOverlay.hide();
  }

  Future<void> createProfile() async {
    try {
      var user = await FirebaseAuth.instance.currentUser;

      final idToken = await user!.getIdToken();
      print(idToken);
      var headers = {
        'Authorization': 'Bearer ' + idToken,
        'Content-Type': 'application/json'
      };
      var request = http.Request(
          'POST', Uri.parse(config_data!.apiUrl! + '/app/user/profile'));
      request.body = json.encode({
        "name": user.providerData[0].displayName ?? "",
        "email": user.email,
        "phone": user.phoneNumber
      });
      request.headers.addAll(headers);

      http.StreamedResponse response = await request.send();

      if (response.statusCode == 200) {
        successToast("User Profile Created .. ");
      } else {
        print(response.reasonPhrase);
      }
    } on SocketException {
      Navigator.of(context).pushNamed('/no-internet-screen').then((value) {
        getHiveData(context);
      });
    } on HttpException {
      errorToast("Error occured in http create Profile");
      setState(() {
        isloading = false;
      });
    } on FormatException {
      setState(() {
        isloading = false;
      });
      errorToast("Error occured in format  create Profile");
    } catch (e) {
      errorToast(e.toString());
      errorToast("exception e in create profile");
      setState(() {
        isloading = false;
      });
    }
  }

  Future<void> shipping_payment_ids() async {
    try {
      var cartTableData = await DatabaseCreator.instance.getCartTableData();
      print(cartTableData);
      var user = await FirebaseAuth.instance.currentUser;

      final idToken = await user!.getIdToken();
      // print(idToken);

      var headers = {'Authorization': 'Bearer ' + idToken};

      await Future.forEach(cartTableData, (cart_row) async {
        Map<String, dynamic> cart_row_map = cart_row as Map<String, dynamic>;
        print(cart_row_map);

        var request = http.Request(
            'GET',
            Uri.parse(config_data!.apiUrl! +
                '/app/store/' +
                cart_row_map['store_id']));

        request.headers.addAll(headers);
        http.StreamedResponse response = await request.send();
        String response_string = await response.stream.bytesToString();
        var parsedResponse = jsonDecode(response_string);
        print(parsedResponse);

        if (response.statusCode == 200) {
          int count = await DatabaseCreator.instance.updateShippingPaymentId(
              cart_row_map['store_id'],
              parsedResponse['payment_method_id'],
              parsedResponse['shipping_method_id']);
          print(count);
        } else {
          print(response.reasonPhrase);
        }
      });
    } on SocketException {
      setState(() {
        isloading = false;
      });

      Navigator.of(context).pushNamed('/no-internet-screen').then((value) {
        getHiveData(context);
      });
    } on HttpException {
      errorToast("Error occured in http shipping_payment_ids");
      setState(() {
        isloading = false;
      });
    } on FormatException {
      setState(() {
        isloading = false;
      });
      errorToast("Error occured in format  shipping_payment_ids");
    } catch (e) {
      errorToast(e.toString());
      errorToast("shipping_payment_ids");
      setState(() {
        isloading = false;
      });
    }
  }

  Future<void> save_customer_ids() async {
    try {
      var cartTableData = await DatabaseCreator.instance.getCartTableData();
      print(cartTableData);
      var user = await FirebaseAuth.instance.currentUser;

      final idToken = await user!.getIdToken();
      // print(idToken);

      var headers = {
        'Authorization': 'Bearer ' + idToken,
        'Content-Type': 'application/json'
      };
      await Future.forEach(cartTableData, (cart_row) async {
        Map<String, dynamic> cart_row_map = cart_row as Map<String, dynamic>;
        print(cart_row_map);

        var request = http.Request(
            'POST', Uri.parse(config_data!.apiUrl! + '/app/user/stores'));

        request.body = json.encode({"store_id": cart_row_map['store_id']});
        request.headers.addAll(headers);

        http.StreamedResponse response = await request.send();
        String response_string = await response.stream.bytesToString();
        var parsedResponse = jsonDecode(response_string);
        print(parsedResponse);

        if (response.statusCode == 200) {
          int count = await DatabaseCreator.instance.updateCustomerId(
              cart_row_map['store_id'], parsedResponse['meta']['customer_id']);
          print(count);
        } else {
          print(response.reasonPhrase);
        }
      });
    } on SocketException {
      setState(() {
        isloading = false;
      });

      Navigator.of(context).pushNamed('/no-internet-screen').then((value) {
        getHiveData(context);
      });
    } on HttpException {
      errorToast("Error occured in http save_customer_ids");
      setState(() {
        isloading = false;
      });
    } on FormatException {
      setState(() {
        isloading = false;
      });
      errorToast("Error occured in format  save_customer_ids");
    } catch (e) {
      errorToast(e.toString());
      errorToast("save_customer_ids");
      setState(() {
        isloading = false;
      });
    }
  }

  Future<void> getcartTotal() async {
    try {
      cart_total = 0;
      shipping_charges = 0;
      discount = 0;
      grand_total = 0;

      setState(() {
        isloading = true;
      });
      var cartTableData = await DatabaseCreator.instance.getCartTableData();
      print(cartTableData);

      var headers = {'Content-Type': 'application/json'};

      await Future.forEach(cartTableData, (cart_row) async {
        Map<String, dynamic> cart_row_map = cart_row as Map<String, dynamic>;
        print(cart_row_map);

        var request = http.Request(
            'GET',
            Uri.parse(config_data!.gatewayApiUrl! +
                '/sales/carts/' +
                cart_row_map['cart_id'].toString() +
                '/?store=' +
                cart_row_map['store_id'] +
                '&query={id,discount,shipping_cost,shipping_method,subtotal,total,store{id,currency_symbol,currency}}' +
                '&apikey=' +
                config_data!.gatewayApiKey.toString()));

        request.headers.addAll(headers);
        http.StreamedResponse response = await request.send();
        String response_string = await response.stream.bytesToString();
        var parsedResponse = jsonDecode(response_string);
        // print(parsedResponse['results'][0]);

        if (response.statusCode == 200) {
          print(parsedResponse['store']['currency']);
          setState(() {
            cart_total = cart_total + double.parse(parsedResponse['subtotal']);
            shipping_charges = shipping_charges +
                double.parse(parsedResponse['shipping_cost'] ?? '0');
            discount =
                discount + double.parse(parsedResponse['discount'] ?? '0');

            grand_total =
                grand_total + double.parse(parsedResponse['total'] ?? '0');
            currency_symbol = parsedResponse['store']['currency_symbol'];
            currency_code = parsedResponse['store']['currency'];
          });
        } else {
          print(response.reasonPhrase);
          setState(() {
            isloading = false;
          });
        }
      });

      setState(() {
        isloading = false;
      });
    } on SocketException {
      setState(() {
        isloading = false;
      });

      Navigator.of(context).pushNamed('/no-internet-screen').then((value) {
        getHiveData(context);
      });
    } on HttpException {
      errorToast("Error occured in http getcartTotal");
      setState(() {
        isloading = false;
      });
    } on FormatException {
      setState(() {
        isloading = false;
      });
      errorToast("Error occured in format  getcartTotal");
    } catch (e) {
      errorToast(e.toString());
      errorToast("getcartTotal");
      setState(() {
        isloading = false;
      });
    }
  }

  // methods for checkout button press

  Future<void> checkout_common_process() async {
    try {
      setState(() {
        checkout_loading = true;
      });
      var cartTableData = await DatabaseCreator.instance.getCartTableData();
      var user = await FirebaseAuth.instance.currentUser;

      final idToken = await user!.getIdToken();

      var headers = {'Content-Type': 'application/x-www-form-urlencoded'};
      await Future.forEach(cartTableData, (cart_row) async {
        Map<String, dynamic> cart_row_map = cart_row as Map<String, dynamic>;
        print(cart_row_map);

        var request = http.Request(
            'POST',
            Uri.parse(config_data!.gatewayApiUrl! +
                '/sales/checkout' +
                '/?store=' +
                cart_row_map['store_id'] +
                '&apikey=' +
                config_data!.gatewayApiKey.toString()));

        request.bodyFields = {
          'cart': cart_row_map['cart_id'],
          'payment_method': cart_row_map['payment_method_id']
        };
        request.headers.addAll(headers);
        http.StreamedResponse response = await request.send();
        String response_string = await response.stream.bytesToString();
        // var parsedResponse = jsonDecode(response_string);
        var parsedResponse = jsonDecode(response_string);
        print(response.statusCode);

        if (response.statusCode == 201 || response.statusCode == 200) {
          // print(parsedResponse['order']['store']['id']);
          print(parsedResponse['order']);

          var user = await FirebaseAuth.instance.currentUser;
          // print(user);

          var collection = FirebaseFirestore.instance.collection('users');
          await collection
              .doc(user!.uid)
              .collection("orders")
              .doc(parsedResponse['order']['id'])
              .set({
            "order_id": parsedResponse['order']['id'],
            "store_id": parsedResponse['order']['store']['id'],
          });

          print("checkout  success");
          Navigator.of(context).pushNamed('/congratulations-screen');
          DatabaseCreator.instance.deleteCartData();
          SharedPreferences preferences = await SharedPreferences.getInstance();
          await preferences.clear();
          setState(() {
            checkout_loading = false;
          });
        } else {
          setState(() {
            checkout_loading = false;
          });
          print(response.reasonPhrase);
        }
      });
    } on SocketException {
      setState(() {
        isloading = false;
      });

      Navigator.of(context).pushNamed('/no-internet-screen').then((value) {
        getHiveData(context);
      });
    } on HttpException {
      errorToast("Error occured in http checkout_common_process");
      setState(() {
        isloading = false;
      });
    } on FormatException {
      setState(() {
        isloading = false;
      });
      errorToast("Error occured in format  checkout_common_process");
    } catch (e) {
      errorToast(e.toString());
      errorToast("checkout_common_process");
      setState(() {
        isloading = false;
      });
    }
  }

  Future<void> stripeIntegration() async {
    try {
      setState(() {
        checkout_loading = true;
      });
      var user = await FirebaseAuth.instance.currentUser;

      final idToken = await user!.getIdToken();

      await user.reload();
      user = await FirebaseAuth.instance.currentUser;

      var headers = {
        'Authorization': 'Bearer ' + idToken,
        'Content-Type': 'application/x-www-form-urlencoded'
      };
      var request = http.Request('POST',
          Uri.parse(config_data!.apiUrl! + '/app/stripe/paymentIntent'));

      // print(grand_total);

      // print(currency_code);
      request.bodyFields = {
        'amount': grand_total.toString(),
        'currency': currency_code.toString()
      };
      request.headers.addAll(headers);

      http.StreamedResponse response = await request.send();
      String response_string = await response.stream.bytesToString();
      // var parsedResponse = jsonDecode(response_string);
      var parsedResponse = jsonDecode(response_string);
      print(parsedResponse!['customer']);
      if (response.statusCode == 200) {
        try {
          await Stripe.instance.initPaymentSheet(
              paymentSheetParameters: SetupPaymentSheetParameters(
            applePay: true,
            googlePay: true,
            style: ThemeMode.light,
            // testEnv: true,
            merchantCountryCode: 'PK',
            merchantDisplayName: self_data!.company_name ?? "",
            customerId: parsedResponse!['customer'],
            paymentIntentClientSecret: parsedResponse!['paymentIntent'],
            customerEphemeralKeySecret: parsedResponse!['ephemeralKey'],
          ));

          await Stripe.instance.presentPaymentSheet();

          checkout_common_process();
        } catch (e) {
          setState(() {
            checkout_loading = false;
          });
          errorToast(e);
        }
      } else {
        setState(() {
          checkout_loading = false;
        });
        errorToast(response.reasonPhrase);
      }
    } on SocketException {
      setState(() {
        isloading = false;
      });

      Navigator.of(context).pushNamed('/no-internet-screen').then((value) {
        getHiveData(context);
      });
    } on HttpException {
      errorToast("Error occured in http stripeIntegration");
      setState(() {
        isloading = false;
      });
    } on FormatException {
      setState(() {
        isloading = false;
      });
      errorToast("Error occured in format  stripeIntegration");
    } catch (e) {
      errorToast(e.toString());
      errorToast("stripeIntegration");
      setState(() {
        isloading = false;
      });
    }
  }

  void complete_checkout() async {
    if (selected_payment_method.method == "cod") {
      checkout_common_process();
    } else if (selected_payment_method.method == "stripe") {
      await stripeIntegration();
      // print(selected_payment_method.method);

      // print(self_data!.company_name);
    } else if (selected_payment_method.method == "cactus_finance") {
      setState(() {
        checkout_loading = true;
      });
      // print('cactus');
      final authorizationEndpoint =
          Uri.parse('https://id.mintfinancialonline.com/auth/o/token/');
      final identifier = selected_payment_method.public_key;
      final secret = selected_payment_method.private_key;
      var client = await oauth2.clientCredentialsGrant(
          authorizationEndpoint, identifier, secret);

      // print(client);

      var cartTableData = await DatabaseCreator.instance.getCartTableData();
      List temp = [];

      cartTableData.forEach((cartRow) {
        temp.add({
          "store_id": cartRow['store_id'],
          "cart_id": cartRow['cart_id'],
          "payment_method_id": cartRow['payment_method_id'],
        });
      });

      var meta_data = {'storegence_marketplace': temp};

      // print(meta_data);

      var headers = {
        'Authorization': 'Bearer ${client.credentials.accessToken}',
        'Content-Type': 'application/json'
      };
      var request = http.Request('POST',
          Uri.parse('https://api.mintfinancialonline.com/cactus/purchases/'));
      request.body =
          json.encode({"amount": grand_total, "meta_data": meta_data});
      request.headers.addAll(headers);

      http.StreamedResponse response = await request.send();
      String response_string = await response.stream.bytesToString();
      var parsedResponse = jsonDecode(response_string);

      if (response.statusCode == 201) {
        // print(parsedResponse['payment_link']);

        await DatabaseCreator.instance.deleteCartData();
        SharedPreferences preferences = await SharedPreferences.getInstance();
        await preferences.clear();
        setState(() {
          checkout_loading = false;
        });

        await urlLaunch(parsedResponse['payment_link']);
        Navigator.of(context).pushNamed('/tab-bar-screen');
      } else {
        print(response.reasonPhrase);
      }
    } else {
      print("else");
    }
  }

  @override
  void initState() {
    getHiveData(context);

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final height = MediaQuery.of(context).size.height;
    final width = MediaQuery.of(context).size.width;
    final size = MediaQuery.of(context).size;
    return Scaffold(
      body: SafeArea(
        child: Container(
          margin: EdgeInsets.symmetric(horizontal: 5, vertical: 5),
          padding: EdgeInsets.only(top: 5, left: 10, right: 10, bottom: 5),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              SizedBox(
                height: size.height * 0.02,
              ),
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  InkWell(
                    onTap: () {
                      Navigator.of(context).pop();
                    },
                    child: Image.asset(
                      back_icon,
                      width: width * 0.09,
                    ),
                  ),
                  GestureDetector(
                    onTap: () {
                      Navigator.of(context).pushNamed('/add-address');
                    },
                    child: Row(
                      children: [
                        Text(
                          'Add Address',
                          style: Theme.of(context).textTheme.subtitle1,
                        ),
                        SizedBox(
                          width: size.width * 0.02,
                        ),
                        Image.asset(
                          location_icon,
                          width: width * 0.09,
                        ),
                      ],
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: size.height * 0.04,
              ),
              Text('Checkout', style: Theme.of(context).textTheme.headline1),
              SizedBox(
                height: size.height * 0.04,
              ),
              Container(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Text(
                          'Address',
                          style: Theme.of(context).textTheme.bodyText2,
                        ),
                        GestureDetector(
                          onTap: () {
                            Navigator.of(context)
                                .pushNamed('/change-address')
                                .then((value) {
                              getHiveData(context);
                            });

                            // _showModalBottomSheetChangeAddressScreen(context);
                          },
                          child: Text(
                            'Change Address',
                            style: Theme.of(context).textTheme.subtitle1!.merge(
                                  const TextStyle(
                                      color: Colors.grey,
                                      fontWeight: FontWeight.normal,
                                      fontSize: 12),
                                ),
                          ),
                        ),
                      ],
                    ),
                    SizedBox(
                      height: size.height * 0.02,
                    ),
                    !isAddresSelected
                        ? GestureDetector(
                            onTap: () {
                              Navigator.of(context)
                                  .pushNamed('/change-address')
                                  .then((value) {
                                getHiveData(context);
                              });
                            },
                            child: Container(
                              height: height * 0.05,
                              width: double.infinity,
                              decoration: BoxDecoration(
                                  borderRadius: BorderRadius.circular(10),
                                  color: Colors.red),
                              child: Center(
                                child: Text(
                                  'Select an address to continue',
                                  style: Theme.of(context)
                                      .textTheme
                                      .subtitle2!
                                      .merge(TextStyle(fontSize: 14)),
                                ),
                              ),
                            ),
                          )
                        : Text(
                            street! +
                                '\n' +
                                city! +
                                " , " +
                                state! +
                                "\n" +
                                country!,
                            style: Theme.of(context).textTheme.subtitle1,
                          ),
                    SizedBox(
                      height: size.height * 0.03,
                    ),
                    const Divider(
                      thickness: 2,
                    ),
                    SizedBox(
                      height: size.height * 0.03,
                    ),
                    Text(
                      'Cart Details',
                      style: Theme.of(context).textTheme.bodyText2,
                    ),
                    SizedBox(
                      height: size.height * 0.02,
                    ),
                    !isloading
                        ? Column(
                            children: [
                              Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceBetween,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  Text(
                                    'Sub Total: ',
                                    style:
                                        Theme.of(context).textTheme.subtitle1,
                                  ),
                                  Text(
                                    currency_symbol! +
                                        " " +
                                        cart_total.toString(),
                                    style:
                                        Theme.of(context).textTheme.subtitle1,
                                  ),
                                ],
                              ),
                              SizedBox(
                                height: size.height * 0.01,
                              ),
                              Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceBetween,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  Text(
                                    'Discount: ',
                                    style:
                                        Theme.of(context).textTheme.subtitle1,
                                  ),
                                  Text(
                                    currency_symbol! +
                                        " " +
                                        discount.toString(),
                                    style:
                                        Theme.of(context).textTheme.subtitle1,
                                  ),
                                ],
                              ),
                              SizedBox(
                                height: size.height * 0.01,
                              ),
                              Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceBetween,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  Text(
                                    'Shipping Charges ',
                                    style:
                                        Theme.of(context).textTheme.subtitle1,
                                  ),
                                  Text(
                                    currency_symbol! +
                                        " " +
                                        shipping_charges.toString(),
                                    style:
                                        Theme.of(context).textTheme.subtitle1,
                                  ),
                                ],
                              ),
                              SizedBox(
                                height: size.height * 0.01,
                              ),
                              Row(
                                mainAxisAlignment:
                                    MainAxisAlignment.spaceBetween,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  Text(
                                    'Grand Total ',
                                    style:
                                        Theme.of(context).textTheme.subtitle1,
                                  ),
                                  Text(
                                    currency_symbol! +
                                        " " +
                                        grand_total.toString(),
                                    style:
                                        Theme.of(context).textTheme.subtitle1,
                                  ),
                                ],
                              ),
                            ],
                          )
                        : Center(child: Text("Loading Cart Details ... ")),
                    Divider(
                      thickness: 2,
                    ),
                    SizedBox(
                      height: size.height * 0.03,
                    ),
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Text(
                          'Payment',
                          style: Theme.of(context).textTheme.bodyText2,
                        ),
                        GestureDetector(
                          onTap: () {
                            Navigator.of(context)
                                .pushNamed('/change-payment-method')
                                .then((value) {
                              getHiveData(context);
                            });
                          },

                          // _showModalBottomSheetChangePaymentMethodScreen(
                          //     context);

                          child: Text(
                            'Change Payment Method',
                            style: Theme.of(context).textTheme.subtitle1!.merge(
                                  // ignore: prefer_const_constructors
                                  TextStyle(
                                      color: Colors.grey,
                                      fontWeight: FontWeight.normal,
                                      fontSize: 12),
                                ),
                          ),
                        ),
                      ],
                    ),
                    SizedBox(
                      height: size.height * 0.03,
                    ),
                    selected_payment_method == null
                        ? GestureDetector(
                            onTap: () {
                              Navigator.of(context)
                                  .pushNamed('/change-payment-method')
                                  .then((value) {
                                getHiveData(context);
                              });
                            },
                            child: Container(
                              height: height * 0.05,
                              width: double.infinity,
                              decoration: BoxDecoration(
                                  borderRadius: BorderRadius.circular(10),
                                  color: Colors.red),
                              child: Center(
                                child: Text(
                                  'Select a payment method',
                                  style: Theme.of(context)
                                      .textTheme
                                      .subtitle2!
                                      .merge(TextStyle(fontSize: 14)),
                                ),
                              ),
                            ),
                          )
                        : Text(
                            selected_payment_method.name.toString(),
                            style: Theme.of(context).textTheme.subtitle1,
                          ),
                    SizedBox(
                      height: size.height * 0.03,
                    ),
                  ],
                ),
              ),
              Spacer(),
              isAddresSelected && selected_payment_method != null
                  ? (!checkout_loading
                      ? ButtonGlobal(
                          text: 'Complete Checkout',
                          onTap: () async {
                            // print(selected_payment_method.private_key);
                            complete_checkout();
                          })
                      : Container(
                          width: double.infinity,
                          decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(10),
                            color: baseColor_1,
                          ),
                          child: Padding(
                            padding: const EdgeInsets.symmetric(
                                vertical: 18.0, horizontal: 100),
                            child: Center(
                              child: Text(
                                'Order Processing  ...',
                                style: Theme.of(context)
                                    .textTheme
                                    .subtitle2!
                                    .merge(TextStyle(fontSize: 14)),
                              ),
                            ),
                          ),
                        ))
                  : Container(),
              SizedBox(
                height: size.height * 0.04,
              ),
            ],
          ),
        ),
      ),
    );
  }
}
