import 'dart:io';

import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';
import 'package:sqflite/sqflite.dart';

class DatabaseCreator {
  static final _dbName = 'storegence_marketplace.db';
  static final _dbVersion = 1;

  DatabaseCreator._privateConstructor();

  static final DatabaseCreator instance = DatabaseCreator._privateConstructor();

  static Database? _database;

  Future<Database?> get database async {
    if (_database != null) {
      return _database;
    } else {
      _database = await _intiateDatabase();
      return _database;
    }
  }

  _intiateDatabase() async {
    Directory directory = await getApplicationDocumentsDirectory();

    print(directory.path);
    String path = join(directory.path, _dbName);
    return await openDatabase(
      path,
      version: _dbVersion,
      onCreate: _onCreate,
    );
  }

  Future _onCreate(Database db, int version) async {
    db.execute(
        "CREATE TABLE cartTable (cart_id VARCHAR,store_id VARCHAR,shipping_method_id VARCHAR,payment_method_id VARCHAR, customer_id VARCHAR,shipping_address_id VARCHAR);");
  }

  Future<void> createOrReturn() async {
    await instance.database;
  }

  Future<int?> checkIfTableExists(String tableName) async {
    Database? db = await instance.database;

    try {
      var dbclient = await db;
      int? count = Sqflite.firstIntValue(
          await dbclient!.rawQuery("SELECT COUNT(*) FROM $tableName"));
      return count;
    } catch (e) {
      return -1;
    }
  }

  Future<int> insert(
    String cart_id,
    String store_id,
  ) async {
    Map<String, dynamic> row = {
      'cart_id': cart_id,
      'store_id': store_id,
    };
    Database? db = await instance.database;
    return await db!.insert("cartTable", row);
  }

  Future<List<Map<String, dynamic>>> getCartTableData() async {
    Database? db = await instance.database;

    return await db!.query(
      "cartTable",
    );
  }

  Future<List<Map<String, dynamic>>> getAllCartTableData() async {
    Database? db = await instance.database;

    return await db!.rawQuery('SELECT * FROM cartTable');
  }

  Future<List<Map<String, dynamic>>> getDataByStoreId(String store_id) async {
    Database? db = await instance.database;

    List<Map<String, dynamic>> data = await db!.query(
      "cartTable",
      where: "store_id = ?",
      whereArgs: [store_id],
    );
    print(data);
    return data;
  }

  Future<int> updateShippingPaymentId(String store_id, String payment_method_id,
      String shipping_method_id) async {
    Database? db = await DatabaseCreator.instance.database;

    Map<String, dynamic> row = {
      'payment_method_id': payment_method_id,
      'shipping_method_id': shipping_method_id,
    };

    // do the update and get the number of affected rows
    int updateCount = await db!
        .update('cartTable', row, where: 'store_id = ?', whereArgs: [store_id]);

    return updateCount;
  }

  Future<int> updateShippingAddressId(
      String store_id, String shipping_address_id) async {
    Database? db = await DatabaseCreator.instance.database;

    Map<String, dynamic> row = {
      'shipping_address_id': shipping_address_id,
    };

    // do the update and get the number of affected rows
    int updateCount = await db!
        .update('cartTable', row, where: 'store_id = ?', whereArgs: [store_id]);

    return updateCount;
  }

  Future<int> updateCustomerId(String store_id, String customer_id) async {
    Database? db = await DatabaseCreator.instance.database;

    Map<String, dynamic> row = {
      'customer_id': customer_id,
    };

    // do the update and get the number of affected rows
    int updateCount = await db!
        .update('cartTable', row, where: 'store_id = ?', whereArgs: [store_id]);

    return updateCount;
  }

  Future<void> deleteCartData() async {
    Database? db = await DatabaseCreator.instance.database;

    // do the update and get the number of affected rows

    db!.delete('cartTable');
    print("deleting cart data");
  }
}
