class ProductsModel {
  ProductsModel({
    required this.count,
    required this.next,
    required this.previous,
    required this.page,
    required this.results,
  });
  late final int count;
  late final String next;
  late final String? previous;
  late final Page page;
  late final List<Results> results;

  ProductsModel.fromJson(Map<String, dynamic> json) {
    count = json['count'];
    next = json['next'];
    previous = json['previous'];
    page = Page.fromJson(json['page']);
    results =
        List.from(json['results']).map((e) => Results.fromJson(e)).toList();
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['count'] = count;
    _data['next'] = next;
    _data['previous'] = previous;
    _data['page'] = page.toJson();
    _data['results'] = results.map((e) => e.toJson()).toList();
    return _data;
  }
}

class Page {
  Page({
    required this.current,
    required this.total,
    required this.size,
    required this.resultSize,
  });
  late final int current;
  late final int total;
  late final int size;
  late final int resultSize;

  Page.fromJson(Map<String, dynamic> json) {
    current = json['current'];
    total = json['total'];
    size = json['size'];
    resultSize = json['result_size'];
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['current'] = current;
    _data['total'] = total;
    _data['size'] = size;
    _data['result_size'] = resultSize;
    return _data;
  }
}

class Results {
  Results({
    required this.id,
    required this.name,
    required this.slug,
    this.stockQuantity,
    this.sku,
    this.upc,
    required this.isActive,
    required this.isFeatured,
    this.description,
    this.shortDescription,
    required this.type,
    required this.price,
    this.specialPrice,
    required this.allowSubscription,
    required this.categories,
    required this.tags,
    required this.productImages,
    this.weight,
    this.dimension,
    required this.productOptions,
    required this.store,
    this.purchaseNote,
    required this.upsellProducts,
    required this.crossSellProducts,
    required this.externalUrl,
    required this.externalButtonText,
    required this.formattedPrices,
    required this.additional,
    required this.createdAt,
    required this.updatedAt,
    required this.url,
  });
  late final String id;
  late final String name;
  late final String slug;
  late final int? stockQuantity;
  late final String? sku;
  late final String? upc;
  late final bool isActive;
  late final bool isFeatured;
  late final String? description;
  late final String? shortDescription;
  late final String type;
  late final String price;
  late final String? specialPrice;
  late final bool allowSubscription;
  late final List<Categories> categories;
  late final List<dynamic> tags;
  late final List<ProductImages> productImages;
  late final String? weight;
  late final String? dimension;
  late final List<dynamic> productOptions;
  late final Store? store;
  late final String? purchaseNote;
  late final List<dynamic> upsellProducts;
  late final List<dynamic> crossSellProducts;
  late final String externalUrl;
  late final String externalButtonText;
  late final FormattedPrices formattedPrices;
  late final Additional additional;
  late final String createdAt;
  late final String updatedAt;
  late final String url;

  Results.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    name = json['name'];
    slug = json['slug'];
    stockQuantity = null;
    sku = null;
    upc = null;
    isActive = json['is_active'];
    isFeatured = json['is_featured'];
    description = null;
    shortDescription = null;
    type = json['type'];
    price = json['price'];
    specialPrice = null;
    allowSubscription = json['allow_subscription'];
    categories = List.from(json['categories'])
        .map((e) => Categories.fromJson(e))
        .toList();
    tags = List.castFrom<dynamic, dynamic>(json['tags']);
    productImages = List.from(json['product_images'])
        .map((e) => ProductImages.fromJson(e))
        .toList();
    weight = null;
    dimension = null;
    productOptions = List.castFrom<dynamic, dynamic>(json['product_options']);
    store = Store.fromJson(json['store']);
    purchaseNote = null;
    upsellProducts = List.castFrom<dynamic, dynamic>(json['upsell_products']);
    crossSellProducts =
        List.castFrom<dynamic, dynamic>(json['cross_sell_products']);
    externalUrl = json['external_url'];
    externalButtonText = json['external_button_text'];
    formattedPrices = FormattedPrices.fromJson(json['formatted_prices']);
    additional = (json['additional'] == null
        ? null
        : Additional.fromJson(json['additional']))!;
    createdAt = json['created_at'];
    updatedAt = json['updated_at'];
    url = json['url'];
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['id'] = id;
    _data['name'] = name;
    _data['slug'] = slug;
    _data['stock_quantity'] = stockQuantity;
    _data['sku'] = sku;
    _data['upc'] = upc;
    _data['is_active'] = isActive;
    _data['is_featured'] = isFeatured;
    _data['description'] = description;
    _data['short_description'] = shortDescription;
    _data['type'] = type;
    _data['price'] = price;
    _data['special_price'] = specialPrice;
    _data['allow_subscription'] = allowSubscription;
    _data['categories'] = categories.map((e) => e.toJson()).toList();
    _data['tags'] = tags;
    _data['product_images'] = productImages.map((e) => e.toJson()).toList();
    _data['weight'] = weight;
    _data['dimension'] = dimension;
    _data['product_options'] = productOptions;
    _data['store'] = store?.toJson();
    _data['purchase_note'] = purchaseNote;
    _data['upsell_products'] = upsellProducts;
    _data['cross_sell_products'] = crossSellProducts;
    _data['external_url'] = externalUrl;
    _data['external_button_text'] = externalButtonText;
    _data['formatted_prices'] = formattedPrices.toJson();
    _data['additional'] = additional.toJson();
    _data['created_at'] = createdAt;
    _data['updated_at'] = updatedAt;
    _data['url'] = url;
    return _data;
  }
}

class Categories {
  Categories({
    required this.id,
    required this.name,
    required this.slug,
    required this.store,
    this.parentCategory,
    required this.createdAt,
    required this.updatedAt,
    required this.url,
  });
  late final String id;
  late final String name;
  late final String slug;
  late final Store store;
  late final Null parentCategory;
  late final String createdAt;
  late final String updatedAt;
  late final String url;

  Categories.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    name = json['name'];
    slug = json['slug'];
    store = Store.fromJson(json['store']);
    parentCategory = null;
    createdAt = json['created_at'];
    updatedAt = json['updated_at'];
    url = json['url'];
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['id'] = id;
    _data['name'] = name;
    _data['slug'] = slug;
    _data['store'] = store.toJson();
    _data['parent_category'] = parentCategory;
    _data['created_at'] = createdAt;
    _data['updated_at'] = updatedAt;
    _data['url'] = url;
    return _data;
  }
}

class Store {
  Store({
    required this.id,
    required this.name,
    required this.slug,
    this.domain,
    required this.isDomainVerified,
    required this.street,
    required this.city,
    required this.state,
    required this.zip,
    required this.country,
    required this.email,
    required this.phone,
    required this.isActive,
    this.logo,
    required this.currency,
    required this.timezone,
    required this.weightUnit,
    required this.dimensionUnit,
    required this.type,
    this.externalLinks,
    required this.geolocation,
    required this.addToCartButtonText,
    required this.currencySymbol,
    required this.allowedShippingZones,
    required this.additional,
    required this.url,
  });
  late final String id;
  late final String name;
  late final String slug;
  late final Null domain;
  late final bool isDomainVerified;
  late final String street;
  late final String city;
  late final String state;
  late final String zip;
  late final String country;
  late final String email;
  late final String phone;
  late final bool isActive;
  late final Null logo;
  late final String currency;
  late final String timezone;
  late final String weightUnit;
  late final String dimensionUnit;
  late final List<String> type;
  late final Null externalLinks;
  late final Geolocation? geolocation;
  late final String addToCartButtonText;
  late final String currencySymbol;
  late final AllowedShippingZones allowedShippingZones;
  late final Additional additional;
  late final String url;

  Store.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    name = json['name'];
    slug = json['slug'];
    domain = null;
    isDomainVerified = json['is_domain_verified'];
    street = json['street'];
    city = json['city'];
    state = json['state'];
    zip = json['zip'];
    country = json['country'];
    email = json['email'];
    phone = json['phone'];
    isActive = json['is_active'];
    logo = null;
    currency = json['currency'];
    timezone = json['timezone'];
    weightUnit = json['weight_unit'];
    dimensionUnit = json['dimension_unit'];
    type = List.castFrom<dynamic, String>(json['type']);
    externalLinks = null;
    geolocation = json['geolocation'] == null
        ? null
        : Geolocation.fromJson(json['geolocation']);
    addToCartButtonText = json['add_to_cart_button_text'];
    currencySymbol = json['currency_symbol'];
    allowedShippingZones =
        AllowedShippingZones.fromJson(json['allowed_shipping_zones']);
    additional = Additional.fromJson(json['additional']);
    url = json['url'];
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['id'] = id;
    _data['name'] = name;
    _data['slug'] = slug;
    _data['domain'] = domain;
    _data['is_domain_verified'] = isDomainVerified;
    _data['street'] = street;
    _data['city'] = city;
    _data['state'] = state;
    _data['zip'] = zip;
    _data['country'] = country;
    _data['email'] = email;
    _data['phone'] = phone;
    _data['is_active'] = isActive;
    _data['logo'] = logo;
    _data['currency'] = currency;
    _data['timezone'] = timezone;
    _data['weight_unit'] = weightUnit;
    _data['dimension_unit'] = dimensionUnit;
    _data['type'] = type;
    _data['external_links'] = externalLinks;
    _data['geolocation'] = geolocation?.toJson();
    _data['add_to_cart_button_text'] = addToCartButtonText;
    _data['currency_symbol'] = currencySymbol;
    _data['allowed_shipping_zones'] = allowedShippingZones.toJson();
    _data['additional'] = additional.toJson();
    _data['url'] = url;
    return _data;
  }
}

class Geolocation {
  Geolocation({
    required this.type,
    required this.coordinates,
  });
  late final String type;
  late final List<double> coordinates;

  Geolocation.fromJson(Map<String, dynamic> json) {
    type = json['type'];
    coordinates = List.castFrom<dynamic, double>(json['coordinates']);
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['type'] = type;
    _data['coordinates'] = coordinates;
    return _data;
  }
}

class AllowedShippingZones {
  AllowedShippingZones({
    required this.PK,
  });
  late final List<String> PK;

  AllowedShippingZones.fromJson(Map<String, dynamic> json) {
    PK = List.castFrom<dynamic, String>(json['PK']);
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['PK'] = PK;
    return _data;
  }
}

class Additional {
  Additional({
    required this.currencySymbol,
    required this.products,
    required this.categories,
    required this.paymentMethods,
    required this.shippingMethods,
    required this.shippingZones,
  });
  late final String currencySymbol;
  late final String products;
  late final String categories;
  late final String paymentMethods;
  late final String shippingMethods;
  late final String shippingZones;

  Additional.fromJson(Map<String, dynamic> json) {
    currencySymbol =
        json['currency_symbol'] == null ? null : json['currency_symbol'];
    products = json['products'];
    categories = json['categories'];
    paymentMethods = json['payment_methods'];
    shippingMethods = json['shipping_methods'];
    shippingZones = json['shipping_zones'];
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['currency_symbol'] = currencySymbol;
    _data['products'] = products;
    _data['categories'] = categories;
    _data['payment_methods'] = paymentMethods;
    _data['shipping_methods'] = shippingMethods;
    _data['shipping_zones'] = shippingZones;
    return _data;
  }
}

class ProductImages {
  ProductImages({
    required this.id,
    required this.path,
    required this.size,
    required this.createdAt,
    required this.updatedAt,
    required this.url,
  });
  late final String id;
  late final String path;
  late final String? size;
  late final String createdAt;
  late final String updatedAt;
  late final String url;

  ProductImages.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    path = json['path'];
    size = json['size'];
    createdAt = json['created_at'];
    updatedAt = json['updated_at'];
    url = json['url'];
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['id'] = id;
    _data['path'] = path;
    _data['size'] = size;
    _data['created_at'] = createdAt;
    _data['updated_at'] = updatedAt;
    _data['url'] = url;
    return _data;
  }
}

class FormattedPrices {
  FormattedPrices({
    required this.price,
    this.specialPrice,
  });
  late final String price;
  late final String? specialPrice;

  FormattedPrices.fromJson(Map<String, dynamic> json) {
    price = json['price'];
    specialPrice = null;
  }

  Map<String, dynamic> toJson() {
    final _data = <String, dynamic>{};
    _data['price'] = price;
    _data['special_price'] = specialPrice;
    return _data;
  }
}
